@@ -181,7 +181,7 @@ totem_display_is_local (void)
 
 	/* Get to the character after the colon */
 	work++;
-	if (work == NULL)
+	if (*work == '\0')
 		return TRUE;
 
 	if (sscanf (work, "%d.%d", &display, &screen) != 2)
@@ -214,21 +214,17 @@ totem_time_to_string (gint64 msecs)
 		/* Translators: This is a time format, like "9:05:02" for 9
 		 * hours, 5 minutes, and 2 seconds. You may change ":" to
 		 * the separator that your locale uses or use "%Id" instead
-		 * of "%d" if your locale uses localized digits. Do not
-		 * translate the "long time format|" part. Remove it from
-		 * the translation.
+		 * of "%d" if your locale uses localized digits.
 		 */
-		return g_strdup_printf (Q_("long time format|%d:%02d:%02d"), hour, min, sec);
+		return g_strdup_printf (C_("long time format", "%d:%02d:%02d"), hour, min, sec);
 	} else {
 		/* minutes:seconds */
 		/* Translators: This is a time format, like "5:02" for 5
 		 * minutes and 2 seconds. You may change ":" to the
 		 * separator that your locale uses or use "%Id" instead of
-		 * "%d" if your locale uses localized digits. Do not
-		 * translate the "short time format|" part. Remove it from
-		 * the translation.
+		 * "%d" if your locale uses localized digits.
 		 */
-		return g_strdup_printf (Q_("short time format|%d:%02d"), min, sec);
+		return g_strdup_printf (C_("short time format", "%d:%02d"), min, sec);
 	}
 
 	return NULL;
