@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
  *
  * The Totem project hereby grant permission for non-gpl compatible GStreamer
  * plugins to be used and distributed together with GStreamer and Totem. This
@@ -148,7 +148,7 @@ impl_activate (TotemPlugin *plugin,
 	/* Try the gnome-settings-daemon version,
 	 * then the gnome-control-center version of things */
 	pi->media_player_keys_proxy = dbus_g_proxy_new_for_name_owner (connection,
-								       "org.gnome.SettingsDaemon.MediaKeys",
+								       "org.gnome.SettingsDaemon",
 								       "/org/gnome/SettingsDaemon/MediaKeys",
 								       "org.gnome.SettingsDaemon.MediaKeys",
 								       NULL);
