@@ -29,7 +29,7 @@
 #include "config.h"
 
 #include <glib-object.h>
-#include <gtk/gtkwindow.h>
+#include <gtk/gtk.h>
 
 #include "totem.h"
 #include "totemobject-marshal.h"
@@ -130,8 +130,8 @@ totem_object_class_init (TotemObjectClass *klass)
 				G_SIGNAL_RUN_LAST,
 				G_STRUCT_OFFSET (TotemObjectClass, metadata_updated),
 				NULL, NULL,
-				totemobject_marshal_VOID__STRING_STRING_STRING,
-				G_TYPE_NONE, 3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+				totemobject_marshal_VOID__STRING_STRING_STRING_UINT,
+				G_TYPE_NONE, 4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT);
 }
 
 static void
@@ -300,7 +300,7 @@ totem_add_to_playlist_and_play (Totem *totem,
 				gboolean add_to_recent)
 {
 	gboolean playlist_changed;
-	guint end;
+	int end;
 
 	totem_signal_block_by_data (totem->playlist, totem);
 
@@ -311,7 +311,7 @@ totem_add_to_playlist_and_play (Totem *totem,
 
 	totem_signal_unblock_by_data (totem->playlist, totem);
 
-	if (playlist_changed)
+	if (playlist_changed && end != -1)
 	{
 		char *mrl, *subtitle;
 
@@ -324,13 +324,24 @@ totem_add_to_playlist_and_play (Totem *totem,
 	}
 }
 
-guint
-totem_get_playlist_length (Totem *totem)
+char *
+totem_get_current_mrl (Totem *totem)
 {
-	return totem_playlist_get_last (totem->playlist) + 1;
+	return totem_playlist_get_current_mrl (totem->playlist, NULL);
 }
 
 guint
+totem_get_playlist_length (Totem *totem)
+{
+	int last;
+
+	last = totem_playlist_get_last (totem->playlist);
+	if (last == -1)
+		return 0;
+	return last + 1;
+}
+
+int
 totem_get_playlist_pos (Totem *totem)
 {
 	return totem_playlist_get_current (totem->playlist);
@@ -349,6 +360,12 @@ totem_get_short_title (Totem *totem)
 	return totem_playlist_get_current_title (totem->playlist, &custom);
 }
 
+void
+totem_set_current_subtitle (Totem *totem, const char *subtitle_uri)
+{
+	totem_playlist_set_current_subtitle (totem->playlist, subtitle_uri);
+}
+
 /**
  * totem_add_sidebar_page:
  * @totem: a #TotemObject
@@ -435,14 +452,16 @@ void
 totem_metadata_updated (TotemObject *totem,
 			const char *artist,
 			const char *title,
-			const char *album)
+			const char *album,
+			guint track_num)
 {
 	g_signal_emit (G_OBJECT (totem),
 		       totem_table_signals[METADATA_UPDATED],
 		       0,
 		       artist,
 		       title,
-		       album);
+		       album,
+		       track_num);
 }
 
 GQuark
