@@ -32,6 +32,8 @@
 #include <gio/gio.h>
 #include <string.h>
 
+#include "eggfileformatchooser.h"
+#include "totem-dnd-menu.h"
 #include "totem-uri.h"
 #include "totem-interface.h"
 #include "video-utils.h"
@@ -42,8 +44,6 @@
 static void ensure_shuffled (TotemPlaylist *playlist, gboolean shuffle);
 static gboolean totem_playlist_add_one_mrl (TotemPlaylist *playlist, const char *mrl, const char *display_name);
 
-typedef gboolean (*PlaylistCallback) (TotemPlaylist *playlist, const char *mrl,
-		gpointer data);
 typedef gboolean (*ClearComparisonFunc) (TotemPlaylist *playlist, GtkTreeIter *iter, gconstpointer data);
 
 static void totem_playlist_clear_with_compare (TotemPlaylist *playlist,
@@ -67,17 +67,6 @@ typedef struct {
 	gpointer user_data;
 } PlaylistForeachContext;
 
-typedef struct {
-	char *mimetype;
-	PlaylistCallback func;
-} PlaylistTypes;
-
-typedef struct {
-	const char *name;
-	const char *suffix;
-	TotemPlParserType type;
-} PlaylistSaveType;
-
 struct TotemPlaylistPrivate
 {
 	GtkBuilder *xml;
@@ -91,12 +80,11 @@ struct TotemPlaylistPrivate
 	GtkActionGroup *action_group;
 	GtkUIManager *ui_manager;
 
-	/* This is a scratch list for when we're removing files */
-	GList *list;
-
 	/* These is the current paths for the file selectors */
 	char *path;
 	char *save_path;
+	guint save_format;
+	GtkWidget *file_chooser;
 
 	/* Shuffle mode */
 	int *shuffled;
@@ -108,6 +96,10 @@ struct TotemPlaylistPrivate
 	GtkTreePath *tree_path;
 	GtkTreeViewDropPosition drop_pos;
 
+	/* This is a scratch list for when we're removing files */
+	GList *list;
+	guint current_to_be_removed : 1;
+
 	guint disable_save_to_disk : 1;
 
 	/* Repeat mode */
@@ -147,11 +139,18 @@ enum {
 	NUM_COLS
 };
 
+typedef struct {
+	const char *name;
+	const char *suffix;
+	TotemPlParserType type;
+} PlaylistSaveType;
+
 static PlaylistSaveType save_types [] = {
-	{".PLS", ".pls", TOTEM_PL_PARSER_PLS},
-	{".M3U", ".m3u", TOTEM_PL_PARSER_M3U},
-	{".M3U (DOS)", ".m3u", TOTEM_PL_PARSER_M3U_DOS},
-	{".XSPF", ".xspf", TOTEM_PL_PARSER_XSPF}
+	{ NULL, NULL, -1 }, /* By extension entry */
+	{ N_("MP3 ShoutCast playlist"), "pls", TOTEM_PL_PARSER_PLS },
+	{ N_("MP3 audio (streamed)"), "m3u", TOTEM_PL_PARSER_M3U },
+	{ N_("MP3 audio (streamed, DOS format)"), "m3u", TOTEM_PL_PARSER_M3U_DOS },
+	{ N_("XML Shareable Playlist"), "xspf", TOTEM_PL_PARSER_XSPF }
 };
 
 static int totem_playlist_table_signals[LAST_SIGNAL];
@@ -246,9 +245,9 @@ totem_playlist_error (char *title, char *reason, TotemPlaylist *playlist)
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK,
-				title);
+				"%s", title);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (error_dialog),
-						  reason);
+						  "%s", reason);
 
 	gtk_container_set_border_width (GTK_CONTAINER (error_dialog), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (error_dialog),
@@ -262,6 +261,88 @@ totem_playlist_error (char *title, char *reason, TotemPlaylist *playlist)
 	gtk_widget_show (error_dialog);
 }
 
+void
+totem_playlist_select_subtitle_dialog(TotemPlaylist *playlist, TotemPlaylistSelectDialog mode)
+{
+	char *subtitle, *current, *path;
+	GFile *file, *dir;
+	TotemPlaylistStatus playing;
+	GtkTreeIter iter;
+
+	if (mode == TOTEM_PLAYLIST_DIALOG_PLAYING) {
+		/* Set subtitle file for the currently playing movie */
+		gtk_tree_model_get_iter (playlist->priv->model, &iter, playlist->priv->current);
+	} else if (mode == TOTEM_PLAYLIST_DIALOG_SELECTED) {
+		/* Set subtitle file in for the first selected playlist item */
+		GList *l;
+
+		l = gtk_tree_selection_get_selected_rows (playlist->priv->selection, NULL);
+		gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
+		g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
+		g_list_free (l);
+	} else {
+		g_assert_not_reached ();
+	}
+
+	/* Look for the directory of the current movie */
+	gtk_tree_model_get (playlist->priv->model, &iter,
+			    FILENAME_COL, &current,
+			    -1);
+
+	if (current == NULL)
+		return;
+
+	path = NULL;
+	file = g_file_new_for_commandline_arg (current);
+	dir = g_file_get_parent (file);
+	g_object_unref (file);
+	if (dir != NULL) {
+		path = g_file_get_path (dir);
+		g_object_unref (dir);
+	}
+
+	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), path);
+	g_free (path);
+
+	if (subtitle == NULL)
+		return;
+
+	gtk_tree_model_get (playlist->priv->model, &iter,
+			    PLAYING_COL, &playing,
+			    -1);
+
+	gtk_list_store_set (GTK_LIST_STORE(playlist->priv->model), &iter,
+			    SUBTITLE_URI_COL, subtitle,
+			    -1);
+
+	if (playing != TOTEM_PLAYLIST_STATUS_NONE) {
+		g_signal_emit (G_OBJECT (playlist),
+			       totem_playlist_table_signals[SUBTITLE_CHANGED], 0,
+			       NULL);
+	}
+
+	g_free(subtitle);
+}
+
+void
+totem_playlist_set_current_subtitle (TotemPlaylist *playlist, const char *subtitle_uri)
+{
+	GtkTreeIter iter;
+
+	if (playlist->priv->current == NULL)
+		return;
+
+	gtk_tree_model_get_iter (playlist->priv->model, &iter, playlist->priv->current);
+
+	gtk_list_store_set (GTK_LIST_STORE(playlist->priv->model), &iter,
+			    SUBTITLE_URI_COL, subtitle_uri,
+			    -1);
+
+	g_signal_emit (G_OBJECT (playlist),
+		       totem_playlist_table_signals[SUBTITLE_CHANGED], 0,
+		       NULL);
+}
+
 /* This one returns a new string, in UTF8 even if the MRL is encoded
  * in the locale's encoding
  */
@@ -391,6 +472,17 @@ drop_cb (GtkWidget        *widget,
 	GList *p, *file_list;
 	guint i;
 
+	if (context->suggested_action == GDK_ACTION_ASK) {
+		context->action = totem_drag_ask (PL_LEN != 0);
+		if (context->action == GDK_ACTION_DEFAULT) {
+			gtk_drag_finish (context, FALSE, FALSE, time);
+			return;
+		}
+	}
+
+	if (context->action == GDK_ACTION_MOVE)
+		totem_playlist_clear (playlist);
+
 	list = g_uri_list_extract_uris ((char *)data->data);
 	file_list = NULL;
 
@@ -461,55 +553,7 @@ drop_cb (GtkWidget        *widget,
 void
 playlist_select_subtitle_action_callback (GtkAction *action, TotemPlaylist *playlist)
 {
-	char *subtitle, *current, *path;
-	GList *l;
-	GFile *file, *dir;
-	TotemPlaylistStatus playing;
-	GtkTreeIter iter;
-
-	l = gtk_tree_selection_get_selected_rows (playlist->priv->selection, NULL);
-	gtk_tree_model_get_iter (playlist->priv->model, &iter, l->data);
-	g_list_foreach (l, (GFunc) gtk_tree_path_free, NULL);
-	g_list_free (l);
-
-	/* Look for the directory of the current movie */
-	gtk_tree_model_get (playlist->priv->model, &iter,
-			    FILENAME_COL, &current,
-			    -1);
-
-	if (current == NULL)
-		return;
-
-	path = NULL;
-	file = g_file_new_for_commandline_arg (current);
-	dir = g_file_get_parent (file);
-	g_object_unref (file);
-	if (dir != NULL) {
-		path = g_file_get_path (dir);
-		g_object_unref (dir);
-	}
-
-	subtitle = totem_add_subtitle (totem_playlist_get_toplevel (playlist), path);
-	g_free (path);
-
-	if (subtitle == NULL)
-		return;
-
-	gtk_tree_model_get (playlist->priv->model, &iter,
-			    PLAYING_COL, &playing,
-			    -1);
-
-	gtk_list_store_set (GTK_LIST_STORE(playlist->priv->model), &iter, 
-			    SUBTITLE_URI_COL, subtitle,
-			    -1);
-
-	if (playing != TOTEM_PLAYLIST_STATUS_NONE) {
-		g_signal_emit (G_OBJECT (playlist),
-			       totem_playlist_table_signals[SUBTITLE_CHANGED], 0,
-			       NULL);
-	}
-
-	g_free(subtitle);
+	totem_playlist_select_subtitle_dialog (playlist, TOTEM_PLAYLIST_DIALOG_SELECTED);
 }
 
 void
@@ -546,7 +590,7 @@ static gboolean
 playlist_show_popup_menu (TotemPlaylist *playlist, GdkEventButton *event)
 {
 	guint button = 0;
-       	guint32 time;
+	guint32 time;
 	GtkTreePath *path;
 	gint count;
 	GtkWidget *menu;
@@ -644,8 +688,7 @@ totem_playlist_set_reorderable (TotemPlaylist *playlist, gboolean set)
 
 		/* Only emit the changed signal if we changed the ->current */
 		path = gtk_tree_path_new_from_indices (i, -1);
-		if (gtk_tree_path_compare (path, playlist->priv->current) == 0)
-		{
+		if (gtk_tree_path_compare (path, playlist->priv->current) == 0) {
 			gtk_tree_path_free (path);
 		} else {
 			gtk_tree_path_free (playlist->priv->current);
@@ -812,6 +855,10 @@ totem_playlist_foreach_selected (GtkTreeModel *model, GtkTreePath *path,
 	ref = gtk_tree_row_reference_new (playlist->priv->model, path);
 	playlist->priv->list = g_list_prepend
 		(playlist->priv->list, (gpointer) ref);
+	if (playlist->priv->current_to_be_removed == FALSE
+	    && playlist->priv->current != NULL
+	    && gtk_tree_path_compare (path, playlist->priv->current) == 0)
+		playlist->priv->current_to_be_removed = TRUE;
 }
 
 static void
@@ -835,110 +882,7 @@ totem_playlist_emit_item_removed (TotemPlaylist *playlist,
 static void
 playlist_remove_files (TotemPlaylist *playlist)
 {
-	GtkTreeSelection *selection;
-	GtkTreeRowReference *ref;
-	gboolean is_selected = FALSE;
-	int next_pos;
-
-	selection = gtk_tree_view_get_selection
-		(GTK_TREE_VIEW (playlist->priv->treeview));
-	if (selection == NULL)
-		return;
-
-	gtk_tree_selection_selected_foreach (selection,
-			totem_playlist_foreach_selected,
-			(gpointer) playlist);
-
-	/* If the current item is to change, we need to keep an static
-	 * reference to it, TreeIter and TreePath don't allow that */
-	if (playlist->priv->current != NULL)
-	{
-		int *indices;
-
-		ref = gtk_tree_row_reference_new (playlist->priv->model,
-				playlist->priv->current);
-		is_selected = gtk_tree_selection_path_is_selected (selection,
-				playlist->priv->current);
-
-		indices = gtk_tree_path_get_indices (playlist->priv->current);
-		next_pos = indices[0];
-
-		gtk_tree_path_free (playlist->priv->current);
-	} else {
-		ref = NULL;
-		next_pos = -1;
-	}
-
-	/* We destroy the items, one-by-one from the list built above */
-	while (playlist->priv->list != NULL)
-	{
-		GtkTreePath *path;
-		GtkTreeIter iter;
-
-		path = gtk_tree_row_reference_get_path
-			((GtkTreeRowReference *)(playlist->priv->list->data));
-		gtk_tree_model_get_iter (playlist->priv->model, &iter, path);
-		gtk_tree_path_free (path);
-
-		totem_playlist_emit_item_removed (playlist, &iter);
-		gtk_list_store_remove (GTK_LIST_STORE (playlist->priv->model), &iter);
-
-		gtk_tree_row_reference_free
-			((GtkTreeRowReference *)(playlist->priv->list->data));
-		playlist->priv->list = g_list_remove (playlist->priv->list,
-				playlist->priv->list->data);
-	}
-	g_list_free (playlist->priv->list);
-	playlist->priv->list = NULL;
-
-	if (is_selected != FALSE)
-	{
-		/* The current item was removed from the playlist */
-		if (next_pos != -1)
-		{
-			char *str;
-			GtkTreeIter iter;
-			GtkTreePath *cur;
-
-			str = g_strdup_printf ("%d", next_pos);
-			cur = gtk_tree_path_new_from_string (str);
-
-			if (gtk_tree_model_get_iter (playlist->priv->model,
-						&iter, cur) == FALSE)
-			{
-				playlist->priv->current = NULL;
-				gtk_tree_path_free (cur);
-			} else {
-				playlist->priv->current = cur;
-			}
-			g_free (str);
-		} else {
-			playlist->priv->current = NULL;
-		}
-
-		playlist->priv->current_shuffled = -1;
-		ensure_shuffled (playlist, playlist->priv->shuffle);
-
-		g_signal_emit (G_OBJECT (playlist),
-				totem_playlist_table_signals[CURRENT_REMOVED],
-				0, NULL);
-	} else {
-		if (ref != NULL)
-		{
-			/* The path to the current item changed */
-			playlist->priv->current =
-				gtk_tree_row_reference_get_path (ref);
-			gtk_tree_row_reference_free (ref);
-		}
-
-		ensure_shuffled (playlist, playlist->priv->shuffle);
-
-		g_signal_emit (G_OBJECT (playlist),
-				totem_playlist_table_signals[CHANGED], 0,
-				NULL);
-	}
-	totem_playlist_update_save_button (playlist);
-	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (playlist->priv->treeview));
+	totem_playlist_clear_with_compare (playlist, NULL, NULL);
 }
 
 void
@@ -956,103 +900,144 @@ playlist_remove_action_callback (GtkAction *action, TotemPlaylist *playlist)
 static void
 totem_playlist_save_playlist (TotemPlaylist *playlist, char *filename, gint active_format)
 {
-	PlaylistSaveType *cur = NULL;
-	guint i;
+	if (active_format == 0)
+		active_format = 1;
 
-	if (active_format > 0)
-		totem_playlist_save_current_playlist_ext (playlist, filename,
-							save_types[active_format - 1].type);
-	else {
-		for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
-			if (g_str_has_suffix (filename, save_types[i].suffix)) {
-				cur = &save_types[i];
-				break;
+	totem_playlist_save_current_playlist_ext (playlist, filename,
+						  save_types[active_format].type);
+}
+
+static char *
+suffix_match_replace (const char *fname, guint old_format, guint new_format)
+{
+	char *ext;
+
+	ext = g_strdup_printf (".%s", save_types[old_format].suffix);
+	if (g_str_has_suffix (fname, ext) != FALSE) {
+		char *no_suffix, *new_fname;
+
+		no_suffix = g_strndup (fname, strlen (fname) - strlen (ext));
+		new_fname = g_strconcat (no_suffix, ".", save_types[new_format].suffix, NULL);
+		g_free (no_suffix);
+		g_free (ext);
+
+		return new_fname;
+	}
+	g_free (ext);
+
+	return NULL;
+}
+
+static void
+format_selection_changed (EggFileFormatChooser *chooser, TotemPlaylist *playlist)
+{
+	guint format;
+
+	format = egg_file_format_chooser_get_format (chooser, NULL);
+
+	if (format != playlist->priv->save_format) {
+		char *fname, *new_fname;
+
+		new_fname = NULL;
+		fname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (playlist->priv->file_chooser));
+
+		if (format == 0) {
+			/* The new format is "By extension" don't touch anything */
+		} else if (playlist->priv->save_format == 0) {
+			guint i;
+
+			for (i = 1; i < G_N_ELEMENTS (save_types); i++) {
+				new_fname = suffix_match_replace (fname, i, format);
+				if (new_fname != NULL)
+					break;
 			}
+		} else {
+			new_fname = suffix_match_replace (fname, playlist->priv->save_format, format);
 		}
-		if (cur == NULL)
-			totem_playlist_error (_("Could not save the playlist"), _("Unknown file extension."), playlist);
-		else
-			totem_playlist_save_current_playlist_ext (playlist, filename, cur->type);
+		if (new_fname != NULL) {
+			char *basename;
+
+			basename = g_path_get_basename (new_fname);
+			g_free (new_fname);
+			gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (playlist->priv->file_chooser), basename);
+			g_free (basename);
+		}
+		playlist->priv->save_format = format;
 	}
 }
 
 static GtkWidget *
-totem_playlist_save_add_format_combo_box (GtkFileChooser *fc)
+totem_playlist_save_add_format_chooser (GtkFileChooser *fc, TotemPlaylist *playlist)
 {
-	GtkWidget *hbox, *label, *combo_box;
+	GtkWidget *format_chooser;
 	guint i;
 
-	hbox = gtk_hbox_new (FALSE, 4);
-	label = gtk_label_new (_("Select playlist format:"));
-	gtk_widget_show (label);
+	format_chooser = egg_file_format_chooser_new ();
 
-	combo_box = gtk_combo_box_new_text ();
-	gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
-			_("By extension"));
-	gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), 0);
-	for (i = 0; i < G_N_ELEMENTS(save_types); i++) {
-		gtk_combo_box_append_text (GTK_COMBO_BOX (combo_box),
-				save_types[i].name);
+	playlist->priv->save_format = 0;
+
+	for (i = 1; i < G_N_ELEMENTS (save_types) ; i++) {
+		egg_file_format_chooser_add_format (
+		    EGG_FILE_FORMAT_CHOOSER (format_chooser), 0, _(save_types[i].name),
+		    "gnome-mime-audio", save_types[i].suffix, NULL);
 	}
-	gtk_widget_show (combo_box);
 
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), combo_box, TRUE, TRUE, 0);
-	gtk_widget_show (hbox);
-	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (fc), hbox);
+	g_signal_connect (format_chooser, "selection-changed",
+			  G_CALLBACK (format_selection_changed), playlist);
 
-	atk_object_add_relationship (gtk_widget_get_accessible (label),
-			ATK_RELATION_LABEL_FOR,
-			gtk_widget_get_accessible (combo_box));
-	atk_object_add_relationship (gtk_widget_get_accessible (combo_box),
-			ATK_RELATION_LABELLED_BY,
-			gtk_widget_get_accessible (label));
+	gtk_file_chooser_set_extra_widget (GTK_FILE_CHOOSER (fc),
+					   format_chooser);
 
-	return combo_box;
+	return format_chooser;
 }
 
 void
 totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 {
-	GtkWidget *fs, *combo_box;
+	GtkWidget *fs, *format_chooser;
 	char *filename;
 	int response;
 
+	g_assert (playlist->priv->file_chooser == NULL);
+
 	fs = gtk_file_chooser_dialog_new (_("Save Playlist"),
-			totem_playlist_get_toplevel (playlist),
-			GTK_FILE_CHOOSER_ACTION_SAVE,
-			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-			GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
-			NULL);
+					  totem_playlist_get_toplevel (playlist),
+					  GTK_FILE_CHOOSER_ACTION_SAVE,
+					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
+					  NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (fs), GTK_RESPONSE_ACCEPT);
 	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (fs), FALSE);
 	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (fs), TRUE);
+
 	/* translators: Playlist is the default saved playlist filename,
 	 * without the suffix */
-	filename = g_strconcat (_("Playlist"), save_types[0].suffix, NULL);
+	filename = g_strconcat (_("Playlist"), ".", save_types[1].suffix, NULL);
 	gtk_file_chooser_set_current_name (GTK_FILE_CHOOSER (fs), filename);
 	g_free (filename);
-	combo_box = totem_playlist_save_add_format_combo_box (GTK_FILE_CHOOSER (fs));
+	format_chooser = totem_playlist_save_add_format_chooser (GTK_FILE_CHOOSER (fs), playlist);
 
-	if (playlist->priv->save_path != NULL)
-	{
+	if (playlist->priv->save_path != NULL) {
 		gtk_file_chooser_set_current_folder_uri (GTK_FILE_CHOOSER (fs),
 				playlist->priv->save_path);
 	}
 
+	playlist->priv->file_chooser = fs;
+
 	response = gtk_dialog_run (GTK_DIALOG (fs));
 	gtk_widget_hide (fs);
 	while (gtk_events_pending())
 		gtk_main_iteration();
 
-	if (response == GTK_RESPONSE_ACCEPT)
-	{
+	if (response == GTK_RESPONSE_ACCEPT) {
 		char *fname;
-		gint active_format;
+		guint active_format;
 
 		fname = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (fs));
-		active_format = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
+		active_format = egg_file_format_chooser_get_format (EGG_FILE_FORMAT_CHOOSER (format_chooser),
+								    fname);
 
+		playlist->priv->file_chooser = NULL;
 		gtk_widget_destroy (fs);
 
 		if (fname == NULL)
@@ -1064,6 +1049,7 @@ totem_playlist_save_files (GtkWidget *widget, TotemPlaylist *playlist)
 		totem_playlist_save_playlist (playlist, fname, active_format);
 		g_free (fname);
 	} else {
+		playlist->priv->file_chooser = NULL;
 		gtk_widget_destroy (fs);
 	}
 }
@@ -1090,8 +1076,7 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 	pos = -2;
 	refs = NULL;
 
-	if (playlist->priv->current != NULL)
-	{
+	if (playlist->priv->current != NULL) {
 		current = gtk_tree_row_reference_new (model,
 				playlist->priv->current);
 	} else {
@@ -1100,8 +1085,7 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 
 	/* Build a list of tree references */
 	paths = gtk_tree_selection_get_selected_rows (selection, NULL);
-	for (l = paths; l != NULL; l = l->next)
-	{
+	for (l = paths; l != NULL; l = l->next) {
 		GtkTreePath *path = l->data;
 		int cur_pos, *indices;
 
@@ -1131,14 +1115,12 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 	else
 		pos = pos - 2;
 
-	for (l = refs; l != NULL; l = l->next)
-	{
+	for (l = refs; l != NULL; l = l->next) {
 		GtkTreeIter *position, cur;
 		GtkTreeRowReference *ref = l->data;
 		GtkTreePath *path;
 
-		if (pos < 0)
-		{
+		if (pos < 0) {
 			position = NULL;
 		} else {
 			char *str;
@@ -1171,8 +1153,7 @@ totem_playlist_move_files (TotemPlaylist *playlist, gboolean direction_up)
 	g_list_free (refs);
 
 	/* Update the current path */
-	if (current != NULL)
-	{
+	if (current != NULL) {
 		gtk_tree_path_free (playlist->priv->current);
 		playlist->priv->current = gtk_tree_row_reference_get_path
 			(current);
@@ -1559,13 +1540,9 @@ totem_playlist_file_changed (GFileMonitor *monitor,
 			     TotemPlaylist *playlist)
 {
 	if (event_type == G_FILE_MONITOR_EVENT_DELETED) {
-		char *uri;
-
-		uri = g_file_get_uri (file);
 		totem_playlist_clear_with_compare (playlist,
 						   (ClearComparisonFunc) totem_playlist_compare_with_monitor,
 						   monitor);
-		g_free (uri);
 	}
 }
 
@@ -1836,78 +1813,139 @@ totem_playlist_clear_with_compare (TotemPlaylist *playlist,
 				   ClearComparisonFunc func,
 				   gconstpointer data)
 {
-	GList *list = NULL, *l;
-	guint num_items, i;
-	gboolean has_items, current_removed;
+	GtkTreeRowReference *ref;
+	int next_pos;
 
-	num_items = PL_LEN;
-	if (num_items == 0)
-		return;
+	ref = NULL;
+	next_pos = -1;
 
-	current_removed = FALSE;
+	if (func == NULL) {
+		GtkTreeSelection *selection;
 
-	for (i = 0; i < num_items; i++)
-	{
-		GtkTreeIter iter;
-		char *index;
+		selection = gtk_tree_view_get_selection
+			(GTK_TREE_VIEW (playlist->priv->treeview));
+		if (selection == NULL)
+			return;
 
-		index = g_strdup_printf ("%d", i);
-		if (gtk_tree_model_get_iter_from_string
-				(playlist->priv->model,
-				 &iter, index) == FALSE)
-		{
+		gtk_tree_selection_selected_foreach (selection,
+						     totem_playlist_foreach_selected,
+						     (gpointer) playlist);
+	} else {
+		guint num_items, i;
+
+		num_items = PL_LEN;
+		if (num_items == 0)
+			return;
+
+		for (i = 0; i < num_items; i++) {
+			GtkTreeIter iter;
+			char *index;
+
+			index = g_strdup_printf ("%d", i);
+			if (gtk_tree_model_get_iter_from_string (playlist->priv->model, &iter, index) == FALSE) {
+				g_free (index);
+				continue;
+			}
 			g_free (index);
-			continue;
-		}
-		g_free (index);
 
-		if ((* func) (playlist, &iter, data) != FALSE)
-		{
-			GtkTreePath *path;
-			GtkTreeRowReference *ref;
+			if ((* func) (playlist, &iter, data) != FALSE) {
+				GtkTreePath *path;
+				GtkTreeRowReference *r;
 
-			path = gtk_tree_path_new_from_indices (i, -1);
-			ref = gtk_tree_row_reference_new
-				(playlist->priv->model, path);
-			list = g_list_prepend (list, ref);
-			gtk_tree_path_free (path);
+				path = gtk_tree_path_new_from_indices (i, -1);
+				r = gtk_tree_row_reference_new (playlist->priv->model, path);
+				if (playlist->priv->current != NULL) {
+					if (gtk_tree_path_compare (path, playlist->priv->current) == 0)
+						playlist->priv->current_to_be_removed = TRUE;
+				}
+				playlist->priv->list = g_list_prepend (playlist->priv->list, r);
+				gtk_tree_path_free (path);
+			}
 		}
+
+		if (playlist->priv->list == NULL)
+			return;
 	}
 
-	has_items = (list != NULL);
+	/* If the current item is to change, we need to keep an static
+	 * reference to it, TreeIter and TreePath don't allow that */
+	if (playlist->priv->current != NULL) {
+		int *indices;
 
-	for (l = list; l != NULL; l = l->next)
+		ref = gtk_tree_row_reference_new (playlist->priv->model,
+				playlist->priv->current);
+		indices = gtk_tree_path_get_indices (playlist->priv->current);
+		next_pos = indices[0];
+
+	}
+
+	/* We destroy the items, one-by-one from the list built above */
+	while (playlist->priv->list != NULL)
 	{
 		GtkTreePath *path;
 		GtkTreeIter iter;
 
-		path = gtk_tree_row_reference_get_path (l->data);
+		path = gtk_tree_row_reference_get_path
+			((GtkTreeRowReference *)(playlist->priv->list->data));
 		gtk_tree_model_get_iter (playlist->priv->model, &iter, path);
-		if (gtk_tree_path_compare (path, playlist->priv->current) == 0)
-			current_removed = TRUE;
+		gtk_tree_path_free (path);
 
 		totem_playlist_emit_item_removed (playlist, &iter);
 		gtk_list_store_remove (GTK_LIST_STORE (playlist->priv->model), &iter);
-		gtk_tree_path_free (path);
-		gtk_tree_row_reference_free (l->data);
+
+		gtk_tree_row_reference_free
+			((GtkTreeRowReference *)(playlist->priv->list->data));
+		playlist->priv->list = g_list_remove (playlist->priv->list,
+				playlist->priv->list->data);
 	}
-	g_list_free (list);
+	g_list_free (playlist->priv->list);
+	playlist->priv->list = NULL;
+
+	if (playlist->priv->current_to_be_removed != FALSE) {
+		/* The current item was removed from the playlist */
+		if (next_pos != -1) {
+			char *str;
+			GtkTreeIter iter;
+			GtkTreePath *cur;
+
+			str = g_strdup_printf ("%d", next_pos);
+			cur = gtk_tree_path_new_from_string (str);
+
+			if (gtk_tree_model_get_iter (playlist->priv->model, &iter, cur) == FALSE) {
+				playlist->priv->current = NULL;
+				gtk_tree_path_free (cur);
+			} else {
+				playlist->priv->current = cur;
+			}
+			g_free (str);
+		} else {
+			playlist->priv->current = NULL;
+		}
 
-	if (has_items != FALSE && current_removed != FALSE) {
 		playlist->priv->current_shuffled = -1;
-
 		ensure_shuffled (playlist, playlist->priv->shuffle);
-		gtk_tree_path_free (playlist->priv->current);
-		playlist->priv->current = NULL;
+
 		g_signal_emit (G_OBJECT (playlist),
 				totem_playlist_table_signals[CURRENT_REMOVED],
 				0, NULL);
-	} else if (has_items != FALSE) {
+	} else {
+		if (ref != NULL) {
+			/* The path to the current item changed */
+			playlist->priv->current =
+				gtk_tree_row_reference_get_path (ref);
+			gtk_tree_row_reference_free (ref);
+		}
+
 		ensure_shuffled (playlist, playlist->priv->shuffle);
+
 		g_signal_emit (G_OBJECT (playlist),
-			       totem_playlist_table_signals[CHANGED], 0,
-			       NULL);
+				totem_playlist_table_signals[CHANGED], 0,
+				NULL);
 	}
+	totem_playlist_update_save_button (playlist);
+	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (playlist->priv->treeview));
+
+	playlist->priv->current_to_be_removed = FALSE;
 }
 
 static gboolean
@@ -2336,19 +2374,19 @@ totem_playlist_set_at_end (TotemPlaylist *playlist)
 	}
 }
 
-guint
+int
 totem_playlist_get_current (TotemPlaylist *playlist)
 {
 	char *path;
 	double index;
 
-	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), 0);
+	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), -1);
 
 	if (playlist->priv->current == NULL)
-		return 0;
+		return -1;
 	path = gtk_tree_path_to_string (playlist->priv->current);
 	if (path == NULL)
-		return 0;
+		return -1;
 
 	index = g_ascii_strtod (path, NULL);
 	g_free (path);
@@ -2356,12 +2394,17 @@ totem_playlist_get_current (TotemPlaylist *playlist)
 	return index;
 }
 
-guint
+int
 totem_playlist_get_last (TotemPlaylist *playlist)
 {
+	guint len = PL_LEN;
+
 	g_return_val_if_fail (TOTEM_IS_PLAYLIST (playlist), -1);
 
-	return PL_LEN - 1;
+	if (len == 0)
+		return -1;
+
+	return len - 1;
 }
 
 void
