@@ -28,7 +28,7 @@
  */
 
 #include <config.h>
-#include <gtk/gtkmessagedialog.h>
+#include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <string.h>
 #include <stdio.h>
@@ -51,7 +51,6 @@ void checkbutton1_toggled_cb (GtkToggleButton *togglebutton, Totem *totem);
 void checkbutton2_toggled_cb (GtkToggleButton *togglebutton, Totem *totem);
 void checkbutton3_toggled_cb (GtkToggleButton *togglebutton, Totem *totem);
 void checkbutton4_toggled_cb (GtkToggleButton *togglebutton, Totem *totem);
-void tvout_toggled_cb (GtkToggleButton *togglebutton, Totem *totem);
 void connection_combobox_changed (GtkComboBox *combobox, Totem *totem);
 void visual_menu_changed (GtkComboBox *combobox, Totem *totem);
 void visual_quality_menu_changed (GtkComboBox *combobox, Totem *totem);
@@ -132,7 +131,7 @@ checkbutton1_toggled_cb (GtkToggleButton *togglebutton, Totem *totem)
 }
 
 static void
-totem_prefs_set_show_visuals (Totem *totem, gboolean value, gboolean warn)
+totem_prefs_set_show_visuals (Totem *totem, gboolean value)
 {
 	GtkWidget *item;
 
@@ -150,21 +149,8 @@ totem_prefs_set_show_visuals (Totem *totem, gboolean value, gboolean warn)
 			"tpw_visuals_size_combobox"));
 	gtk_widget_set_sensitive (item, value);
 
-	if (warn == FALSE)
-	{
-		bacon_video_widget_set_show_visuals
-			(BACON_VIDEO_WIDGET (totem->bvw), value);
-		return;
-	}
-
-	if (bacon_video_widget_set_show_visuals
-			(BACON_VIDEO_WIDGET (totem->bvw), value) == FALSE)
-	{
-		totem_action_info (_("The change of this setting will only "
-					"take effect for the next movie, or "
-					"when Totem is restarted."),
-				totem);
-	}
+	bacon_video_widget_set_show_visuals
+		(BACON_VIDEO_WIDGET (totem->bvw), value);
 }
 
 void
@@ -185,7 +171,7 @@ checkbutton2_toggled_cb (GtkToggleButton *togglebutton, Totem *totem)
 		}
 	}
 
-	totem_prefs_set_show_visuals (totem, value, TRUE);
+	totem_prefs_set_show_visuals (totem, value);
 }
 
 void
@@ -212,22 +198,6 @@ checkbutton4_toggled_cb (GtkToggleButton *togglebutton, Totem *totem)
 			       value, NULL);
 }
 
-void
-tvout_toggled_cb (GtkToggleButton *togglebutton, Totem *totem)
-{
-	TvOutType type;
-	gboolean value;
-
-	value = gtk_toggle_button_get_active (togglebutton);
-	if (value == FALSE)
-		return;
-
-	type = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (togglebutton),
-				"tvout_type"));
-	bacon_video_widget_set_tv_out
-		(BACON_VIDEO_WIDGET (totem->bvw), type);
-}
-
 static void
 deinterlace_changed_cb (GConfClient *client, guint cnxn_id,
 		GConfEntry *entry, Totem *totem)
@@ -509,13 +479,25 @@ totem_setup_preferences (Totem *totem)
 	GtkWidget *menu;
 	GtkAction *action;
 	gboolean show_visuals, auto_resize, is_local, deinterlace, lock_screensaver_on_audio;
-	int connection_speed, i;
+	int connection_speed;
+	guint i, hidden;
 	char *visual, *font, *encoding;
 	GList *list, *l;
 	BaconVideoWidgetAudioOutType audio_out;
 	GConfValue *value;
 	GObject *item;
 
+	static struct {
+		const char *name;
+		BaconVideoWidgetVideoProperty prop;
+		const char *label;
+	} props[4] = {
+		{ "tpw_contrast_scale", BVW_VIDEO_CONTRAST, "tpw_contrast_label" },
+		{ "tpw_saturation_scale", BVW_VIDEO_SATURATION, "tpw_saturation_label" },
+		{ "tpw_bright_scale", BVW_VIDEO_BRIGHTNESS, "tpw_brightness_label" },
+		{ "tpw_hue_scale", BVW_VIDEO_HUE, "tpw_hue_label" }
+	};
+
 	g_return_if_fail (totem->gc != NULL);
 
 	is_local = totem_display_is_local ();
@@ -552,7 +534,6 @@ totem_setup_preferences (Totem *totem)
 
 	/* Boldify some labels */
 	totem_interface_boldify_label (totem->xml, "tpw_network_label");
-	totem_interface_boldify_label (totem->xml, "tpw_tvout_label");
 	totem_interface_boldify_label (totem->xml, "tpw_text_subtitles_label");
 	totem_interface_boldify_label (totem->xml, "tpw_display_label");
 	totem_interface_boldify_label (totem->xml, "tpw_visuals_label");
@@ -598,7 +579,7 @@ totem_setup_preferences (Totem *totem)
 	g_signal_handlers_disconnect_by_func (item, checkbutton2_toggled_cb, totem);
 	gtk_toggle_button_set_active
 		(GTK_TOGGLE_BUTTON (item), show_visuals);
-	totem_prefs_set_show_visuals (totem, show_visuals, FALSE);
+	totem_prefs_set_show_visuals (totem, show_visuals);
 	g_signal_connect (item, "toggled", G_CALLBACK (checkbutton2_toggled_cb), totem);
 
 	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/show_vfx",
@@ -633,8 +614,7 @@ totem_setup_preferences (Totem *totem)
 	item = gtk_builder_get_object (totem->xml, "tpw_visuals_type_combobox");
 
 	i = 0;
-	for (l = list; l != NULL; l = l->next)
-	{
+	for (l = list; l != NULL; l = l->next) {
 		const char *name = l->data;
 
 		gtk_combo_box_append_text (GTK_COMBO_BOX (item), name);
@@ -653,29 +633,28 @@ totem_setup_preferences (Totem *totem)
 	item = gtk_builder_get_object (totem->xml, "tpw_visuals_size_combobox");
 	gtk_combo_box_set_active (GTK_COMBO_BOX (item), i);
 
-	/* Brightness */
-	item = gtk_builder_get_object (totem->xml, "tpw_bright_scale");
-	i = bacon_video_widget_get_video_property (totem->bvw,
-			BVW_VIDEO_BRIGHTNESS);
-	gtk_range_set_value (GTK_RANGE (item), (gdouble) i);
+	/* Brightness and all */
+	hidden = 0;
+	for (i = 0; i < G_N_ELEMENTS (props); i++) {
+		int value;
+		item = gtk_builder_get_object (totem->xml, props[i].name);
+		value = bacon_video_widget_get_video_property (totem->bvw,
+							       props[i].prop);
+		if (value >= 0)
+			gtk_range_set_value (GTK_RANGE (item), (gdouble) value);
+		else {
+			gtk_range_set_value (GTK_RANGE (item), (gdouble) 65535/2);
+			gtk_widget_hide (GTK_WIDGET (item));
+			item = gtk_builder_get_object (totem->xml, props[i].label);
+			gtk_widget_hide (GTK_WIDGET (item));
+			hidden++;
+		}
+	}
 
-	/* Contrast */
-	item = gtk_builder_get_object (totem->xml, "tpw_contrast_scale");
-	i = bacon_video_widget_get_video_property (totem->bvw,
-			BVW_VIDEO_CONTRAST);
-	gtk_range_set_value (GTK_RANGE (item), (gdouble) i);
-
-	/* Saturation */
-	item = gtk_builder_get_object (totem->xml, "tpw_saturation_scale");
-	i = bacon_video_widget_get_video_property (totem->bvw,
-			BVW_VIDEO_SATURATION);
-	gtk_range_set_value (GTK_RANGE (item), (gdouble) i);
-
-	/* Hue */
-	item = gtk_builder_get_object (totem->xml, "tpw_hue_scale");
-	i = bacon_video_widget_get_video_property (totem->bvw,
-			BVW_VIDEO_HUE);
-	gtk_range_set_value (GTK_RANGE (item), (gdouble) i);
+	if (hidden == G_N_ELEMENTS (props)) {
+		item = gtk_builder_get_object (totem->xml, "tpw_bright_contr_vbox");
+		gtk_widget_hide (GTK_WIDGET (item));
+	}
 
 	/* Sound output type */
 	item = gtk_builder_get_object (totem->xml, "tpw_sound_output_combobox");
@@ -749,52 +728,6 @@ totem_setup_preferences (Totem *totem)
 
 }
 
-void
-totem_preferences_tvout_setup (Totem *totem)
-{
-	GObject *item;
-	TvOutType type;
-	const char *name;
-
-	type = bacon_video_widget_get_tv_out (totem->bvw);
-	switch (type)
-	{
-	case TV_OUT_NONE:
-		name = "tpw_notvout_radio_button";
-		break;
-	case TV_OUT_NVTV_PAL:
-		name = "tpw_nvtvpalmode_radio_button";
-		break;
-	case TV_OUT_NVTV_NTSC:
-		name = "tpw_nvtvntscmode_radio_button";
-		break;
-	default:
-		g_assert_not_reached ();
-		name = NULL;
-	}
-
-	item = gtk_builder_get_object (totem->xml, name);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (item), TRUE);
-
-	item = gtk_builder_get_object (totem->xml, "tpw_notvout_radio_button");
-	g_object_set_data (item, "tvout_type",
-			GINT_TO_POINTER (TV_OUT_NONE));
-	gtk_widget_set_sensitive (GTK_WIDGET (item), 
-			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NONE));
-
-	item = gtk_builder_get_object (totem->xml, "tpw_nvtvpalmode_radio_button");
-        g_object_set_data (item, "tvout_type",
-                GINT_TO_POINTER (TV_OUT_NVTV_PAL));
-	gtk_widget_set_sensitive (GTK_WIDGET (item), 
-			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NVTV_PAL));
-
-	item = gtk_builder_get_object (totem->xml, "tpw_nvtvntscmode_radio_button");
-	g_object_set_data (item, "tvout_type",
-                GINT_TO_POINTER (TV_OUT_NVTV_NTSC));
-	gtk_widget_set_sensitive (GTK_WIDGET (item), 
-			bacon_video_widget_fullscreen_mode_available (totem->bvw, TV_OUT_NVTV_NTSC));
-}
-
 void
 totem_preferences_visuals_setup (Totem *totem)
 {
